!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2019 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_initialization.f90
!>
!> \brief Initialize variables
!>
!> See \subpage cs_user_initialization for examples.
!>
!> This subroutine is called at beginning of the computation
!> (restart or not) before the loop time step.
!>
!> This subroutine enables to initialize or modify (for restart)
!> unkown variables and time step values.
!>
!> Modification of the behaviour law of physical quantities (rom, viscl,
!> viscls, cp) is not done here. It is the purpose of the user subroutine
!> \ref cs_user_physical_properties
!>
!> \c rom and \c viscl values are equal to \c ro0 and \c viscl0 or initialize
!> by reading the restart file.
!> Variables diffusivity and specific heat (when they are defined) have no value
!> except if they are read from a restart file.
!>
!> \par cs_user_initialization_cell_id Cells identification
!>
!> Cells may be identified using the \ref getcel subroutine.
!> The syntax of this subroutine is described in the
!> \ref cs_user_boundary_conditions subroutine,
!> but a more thorough description can be found in the user guide.
!>
!> Field cell values arrays can be retrieved using the appropriate access
!> functions as described \ref field "here".
!>
!> Example of field ids:
!> - Density:                        \c irom
!> - Dynamic molecular viscosity:    \c iviscl
!> - Turbulent viscosity:            \c ivisct
!> - Specific heat:                  \c icp
!> - Diffusivity(lambda):            \c field_get_key_int(ivarfl(isca(iscal)),
!>                                      kivisl, ...)
!>
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_user_f_initialization &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use parall
use period
use ppppar
use ppthch
use coincl
use cpincl
use ppincl
use atincl
use ctincl
use ppcpfu
use cs_coal_incl
use cs_fuel_incl
use mesh
use field
use turbomachinery
use user_module

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)

! Local variables


integer, allocatable, dimension(:) :: lstelt


!variables KD 18/3/21

integer iel, kk, k, j, i, ii, numplxy, m
integer nrans, ix, iy, iz, z, rk_id
real dx, dz
parameter (nrans=iu_Ny)  
double precision u_2d(3,nrans), k_2d(nrans), omg_2d(nrans), pr_2d(nrans)
double precision dy(iu_Ny), ypos(iu_Ny), sumdy(iu_Ny),xktot,xwtot,xxu1,xxu2
!real, dimension(iu_Ny) :: sommeR
double precision deltavol, xk, xe, Us, kappac
double precision, dimension(3) :: xup
double precision, parameter :: d2s3 = 2.d0/3.d0
double precision, dimension(:), pointer :: cvar_pr, cvara_pr
double precision, dimension(:), pointer :: cvar_k, cvara_k
double precision, dimension(:), pointer :: cvar_w, cvara_w
double precision, dimension(:,:), pointer :: cvar_vel, cvara_vel
! Friess 15/4/21
double precision, dimension(:), pointer :: psiomg
double precision, dimension(:), pointer :: xk_moy
double precision, dimension(:), pointer :: omega_moy
double precision, dimension(:), pointer :: rk
double precision, dimension(:,:), pointer :: uifujf_moy
double precision, dimension(:,:), pointer :: uif_moy
double precision, dimension(:,:), pointer :: xyzordre
! end Friess 15/4/21


!===============================================================================

! Initialization
!===============================================================================

allocate(lstelt(ncel)) ! temporary array for cells selection

! Lecture des donnees RANS (pre-calcul)
! et association entre mailles 3D et leur equivalent 2D

! Lire les resultats 2D (U1, U2, U3, k, w)
! je fais commencer k a 5 car les 4 premieres lignes des fichiers sont du texte

Open(1,file="results_fluid_domain.velocity.42.00001")
do k=1,4
 read(1,*) ! on ignore les 4 premières lignes
enddo
do kk=1,3 ! composantes vitesse
 do k=1,nrans 
  read(1,*) u_2d(kk,k) 
 enddo ! k
enddo ! kk

Open(2,file="results_fluid_domain.k.42.00001")
do k=1,4
 read(2,*)
enddo
do k=1,nrans
 read(2,*) k_2d(k)
enddo

Open(3,file="results_fluid_domain.omega.42.00001")
do k=1,4
 read(3,*)
enddo
do k=1,nrans
 read(3,*) omg_2d(k)
enddo

Open(4,file="results_fluid_domain.pressure.42.00001")
do k=1,4
 read(4,*)
enddo
do k=1,nrans
 read(4,*) pr_2d(k)
enddo

close(1)
close(2)
close(3)
close(4)

! Determiner, pour chaque maille du domaine 3D, le numero de la maille 
! du domaine 2D, qui lui correspond
! recuperer la numerotation globale
! en deduire l'ordre en x, y et z
! en deduire l'ordre dans le plan xy (pour correspondance avec data RANS)
! affectation des valeurs 

call field_get_val_s(ivarfl(ipr), cvar_pr)
call field_get_val_s(ivarfl(iomg), cvar_w)
call field_get_val_s(ivarfl(ik), cvar_k)
call field_get_val_v(ivarfl(iu), cvar_vel)
! Friess 15/4/21
!if (ihtles.eq.1) then 
 call field_get_val_s_by_name("psiomg", psiomg)
 call field_get_val_s_by_name("k_m", xk_moy)
 call field_get_val_s_by_name("omega_m", omega_moy)
 call field_get_val_v_by_name('uifujf_moy',uifujf_moy)
 call field_get_val_v_by_name('uif_moy',uif_moy)
!endif
call field_get_val_s_by_name("hybrid_energy_ratio", rk)
call field_get_val_v_by_name('xyzordre',xyzordre)
call field_get_val_prev_s(ivarfl(ik), cvara_pr)
call field_get_val_prev_v(ivarfl(iu), cvara_vel)
call field_get_val_prev_s(ivarfl(ik), cvara_k)
call field_get_val_prev_s(ivarfl(iomg), cvara_w)

! end Friess 15/4/21

! Détermination de l'ordre en x,y et z de chaque maille selon les proprietes
! du maillage 

dx=ru_Lx/float(iu_Nx) !dfloat ne fonctionne pas
dz=ru_Lz/float(iu_Nz)

! Friess 9/4/21 :
! Les positions en x et z seront faciles a obtenir. 
! Pour y, on doit ruser un peu 
! 1) On calcule la somme des 1.2^i
dy(1)=1.d0
sumdy(1)=1.d0
do ii=2,iu_Ny
 dy(ii)=ru_prog**(ii-1) ! taille en y de la maille
 sumdy(ii)=sumdy(ii-1)+dy(ii) ! contient la somme des tailles de mailles de 1 a ii
enddo
! sumdy contient la somme des tailles de mailles
! On re-scale 
dy=dy*ru_Ly/sumdy(iu_Ny) ! ecriture vectorielle
sumdy=sumdy*ru_Ly/sumdy(iu_Ny) ! ecriture vectorielle
! 2) on stocke les positions des centres de mailles dans ypos
ypos(1) = dy(1)/2.d0
do ii=2,iu_Ny
 ypos(ii)=sumdy(ii-1)+dy(ii)/2.d0
enddo
! Friess 22/4/21 : sauvegarde des coordonnees pour le post-traitement
! (ca fait gagner un peu de temps)
if (irangp.eq.0) then
 open(11,file='coordonnees.dat')
  write(11,*) iu_Nx, iu_Ny, iu_Nz 
  do ii=1,iu_Nx
   write(11,*) (float(ii)-0.5d0)*dx
  enddo
  do ii=1,iu_Ny
    write(11,*) ypos(ii)
  enddo
  do ii=1,iu_Nz
   write(11,*) (float(ii)-0.5d0)*dz
  enddo
 close(11)
endif
! end Friess 22/4/21 : sauvegarde des coordonnees pour le post-traitement

do iel=1,ncel
 ix = ceiling(xyzcen(1,iel)/dx)  
 iz = ceiling(xyzcen(3,iel)/dz)
! on procede iterativement pour iy 
 ii=1
 do while (xyzcen(2,iel).gt.1.1d0*ypos(ii)) 
  ii=ii+1
 enddo
 iy=ii
! On stocke ix, iy, iz dans un champ pour le post-traitement
xyzordre(1,iel) = ix
xyzordre(2,iel) = iy
xyzordre(3,iel) = iz

 numplxy=iy

 
! on genere des fluctuations gaussiennes pour initialiser la vitesse
 
 xktot = k_2d(numplxy)
 xwtot = omg_2d(numplxy)
 xe = cmu*xktot*xwtot
 deltavol = volume(iel)**(1./3.)
 kappac = pii/deltavol
 rk(iel) = 1.d0/ru_beta0*(kappac*xktot**(3./2.)/xe)**(-2./3.)
 rk(iel)=min(1.d0,rk(iel))
 rk(iel)=max(0.d0,rk(iel))
 
 do ii=1,3
  call random_number(xxu1)
  call random_number(xxu2)
  xup(ii)=sqrt(-2.d0*log(xxu1))*cos(2.d0*pi*xxu2)
 enddo
 cvar_vel(1,iel) = u_2d(1,numplxy)+xup(1)*d2s3*sqrt(xktot*(1.d0-rk(iel)))
 cvar_vel(2,iel) = u_2d(2,numplxy)+xup(2)*d2s3*sqrt(xktot*(1.d0-rk(iel)))
 cvar_vel(3,iel) = u_2d(3,numplxy)+xup(3)*d2s3*sqrt(xktot*(1.d0-rk(iel)))
 ! Friess 22/4/21
 do ii=1,3
  uif_moy(ii,iel)= u_2d(ii,numplxy)
 enddo
 ! Friess 22/4/21
 ! Friess 22/4/21 : k et omega deviennent k_sfs et omega_sfs
 ! + initialisation k_m et omega_m
 cvar_pr(iel) = pr_2d(numplxy)
 cvar_k(iel) = k_2d(numplxy)*rk(iel)
 cvar_w(iel) = omg_2d(numplxy)/rk(iel)
 ! On initialise psiomg "a la louche" pour le premier pas de temps
  psiomg(iel)=7.9d-2/(ru_cmu*0.495+rk(iel)*(7.9d-2-ru_cmu*0.495))
  psiomg(iel)=max(1.d0,psiomg(iel))
! end Friess 22/4/21 : k et omega deviennent k_sfs et omega_sfs
 xk_moy(iel) = cvar_k(iel)
 omega_moy(iel) = cvar_w(iel)
 uifujf_moy(1,iel) =  cvar_vel(1,iel)*cvar_vel(1,iel) 
 uifujf_moy(2,iel) =  cvar_vel(2,iel)*cvar_vel(2,iel) 
 uifujf_moy(3,iel) =  cvar_vel(3,iel)*cvar_vel(3,iel) 
 uifujf_moy(4,iel) =  cvar_vel(1,iel)*cvar_vel(2,iel) 
 uifujf_moy(6,iel) =  cvar_vel(1,iel)*cvar_vel(3,iel) 
 uifujf_moy(5,iel) =  cvar_vel(2,iel)*cvar_vel(3,iel) 
! A toute fin utile, on remplit aussi les "cvara" (valeurs des variables au 
! pas de temps "precedent"
do ii=1,3
cvara_vel(ii,iel)=cvar_vel(ii,iel)
enddo
cvara_k(iel)=cvar_k(iel)
cvara_w(iel)=cvar_w(iel)
cvara_pr(iel)=cvar_pr(iel)
enddo

call field_get_id('hybrid_energy_ratio',rk_id)

!write(*,*) ntmabs, rk_id
!--------
! Formats
!--------

!----
! End
!----

deallocate(lstelt) ! temporary array for cells selection

return
end subroutine cs_user_f_initialization
